#ifndef _ASCIIWriter_h
#define _ASCIIWriter_h



#include "Writer.h"

class ostream;



class ASCIIWriter : public Writer
{
public:
	
		// primitive "builtin" value types
	virtual void writeChar(const char);
	virtual void writeUChar(unsigned char);
	virtual void writeCharPtr(const char*);
	virtual void writeUCharPtr(const unsigned char*);
	virtual void writeLong(long);
	virtual void writeULong(unsigned long);
	virtual void writeShort(short);
	virtual void writeUShort(unsigned short);
	virtual void writeInt(int);
	virtual void writeUInt(unsigned int);
	virtual void writeFloat(float);
	virtual void writeDouble(double);
	virtual void writeBool(bool);
	
		// non-primitive value types not under framework control
	virtual void writeString(const String*);
	virtual void writeString(const String&);
	virtual void writeList(const List*);
	virtual void writeList(const List&);
	virtual void writeStack(const Stack*);
	virtual void writeStack(const Stack&);

		// non-primitive value types under framework control
	virtual void writeValue(const Value&);
	virtual void writeValue(const Value*);

		// references to objects
	virtual void writeObject(const Object*);
	virtual void writeObject(const Object&);
	virtual void writeRoot(const Object*);
	virtual void writeRoot(const Object&);
	
		// ctor, dtor, meta
	ASCIIWriter(ostream&);

private:

	virtual bool wasVisited(const Object*);
	virtual const char* indent();

		// implementation state
	int valueIndent;
	int objectIndent;
	ostream& buffer;
	List* visited;
	Stack* toVisit;
	
};



#endif

