#include "List.h"



class Cons
{
public:
	Cons* pNext;
	Cons* pPrevious;
	const Object* pObject;
	bool isValid;
	int count;
};



const Object* List::item( Cons* pThis )
{
	return nextValid( pThis )->pObject;
}


const Object* List::first()
{
	Cons* pCons = headCons();
	const Object* pObject = item(pCons);
	drop( pCons );
	return pObject;
}


const Object* List::last()
{
	Cons* pCons = previousValid(pTail);
	if ( pCons == null ) return null;
	take( pCons );
	const Object* pObject = item(pCons);
	drop( pCons );
	return pObject;
}


bool List::isEmpty() const
{
	return pHead == pTail;
}


bool List::contains(const Object* pObject)
{
	Cons* pCons = find( pObject );
	drop( pCons );
	return pCons != null;
}

unsigned long List::size() const
{
	unsigned long s = 0;

	for ( Cons* cons = pHead; cons = cons->pNext; )
	{
		if ( cons->isValid ) s++;
	}
	
	return s;
}

Cons* List::insert( Cons* pCons, const Object* pObject )
{
	Cons* pNext = nextValid( pCons );
	Cons* pPrevious = previousValid( pCons );

	drop( pCons );

	pCons = new Cons;

	pCons->pPrevious = pPrevious;
	pCons->pNext = pNext;
	pCons->pObject = pObject;
	pCons->isValid = true;
	pCons->count = 0;

	pNext->pPrevious = pCons;

	if ( pPrevious == null ) pHead = pCons;
	else pPrevious->pNext = pCons;

	take( pNext );
	return pNext;
}


Cons* List::remove( Cons* pCons )
{
	Cons* pNext = nextValid( pCons );
	if ( pNext == pTail ) return pTail;
	drop( pCons );

	take( pNext );
	pNext->isValid = false;

	pCons = nextValid( pNext );
	if ( pNext == pHead ) pHead = pCons;
	drop( pNext );

	take( pCons );
	return pCons;
}


void List::prepend( const Object* pObject )
{
	Cons* pCons = headCons();
	pCons = insert( pCons, pObject );
	drop( pCons );
}


void List::append( const Object* pObject )
{
	Cons* pCons = tailCons();
	pCons = insert( pCons, pObject );
	drop( pCons );
}


void List::remove( const Object* pObject )
{
	Cons* pCons = find( pObject );
	pCons = remove( pCons );
	drop( pCons );
}


void List::removeFirst()
{
	Cons* pCons = headCons();
	pCons = remove( pCons );
	drop( pCons );
}


void List::removeLast()
{
	Cons* pCons = previousValid(pTail);
	if ( pCons == null ) return;
	take( pCons );
	pCons = remove( pCons );
	drop( pCons );
}


Cons* List::headCons()
{
	take( pHead );
	return pHead;
}


Cons* List::tailCons()
{
	take( pTail );
	return pTail;
}


Cons* List::next( Cons* pCons )
{
	Cons* pNext = nextValid( pCons->pNext );

	if ( pNext != null ) this->take( pNext );
	drop( pCons );

	return pNext;
}


Cons* List::previous( Cons* pCons )
{
	Cons* pPrevious = previousValid( pCons );

	if ( pPrevious != null ) take( pPrevious );
	drop( pCons );

	return pPrevious;
}


Cons* List::find(const Object* pObject)
{
	Cons* pCons = null;
	for ( pCons = headCons(); pCons != null; pCons = next( pCons ) )
	{
		if ( pCons->pObject == pObject ) break;
	}

	return pCons;
}


void List::take( Cons* pTake )
{
	pTake->count++;
}


void List::drop( Cons* pDrop )
{
    if ( pDrop == null ) return;

	if ( ! pDrop->isValid && pDrop->count == 1 )
	{
		Cons* pNext = nextValid( pDrop->pNext );
		Cons* pPrevious = previousValid( pDrop );
		if ( pNext != null ) pNext->pPrevious = pPrevious;
		if ( pPrevious != null ) pPrevious->pNext = pNext;
		delete pDrop;
	}
	else
	{
		pDrop->count--;
	}
}


List::List()
{
	pHead = new Cons;

	pHead->pNext = null;
	pHead->pPrevious = null;
	pHead->pObject = null;
	pHead->isValid = true;
	pHead->count = 0;

	pTail = pHead;
}


List::~List()
{
	for ( Cons* pNext, * pThis = pHead; pThis != null; pThis = pNext )
	{
		pNext = pThis->pNext;
		delete pThis;
	}
}


Cons* List::nextValid( Cons* pCons )
{
	while ( pCons && ! pCons->isValid ) pCons = pCons->pNext;
	return pCons;
}


Cons* List::previousValid( Cons* pCons )
{
	if ( pCons != null ) pCons = pCons->pPrevious;
	while ( pCons != null && ! pCons->isValid ) pCons = pCons->pPrevious;
	return pCons;
}

